#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon May 23 16:04:55 2022

@author: benjamin
"""

import numpy as np
from maedeep._areafunction import (AreaFunction)
from maedeep._waveguide import Waveguide
from maedeep._contour import extract_task_variables
from .modeltools import check_model, contour_to_area_function, vector_to_contours

def area_to_formant(area_function):
    wvg = Waveguide("area_function", area_function)
    wvg.computetransferfunction()
    return wvg.computeformants()

def area_to_transfer_function(area_function, df=50):
    wvg = Waveguide("area_function", area_function)
    return wvg.computetransferfunction(df=df)
    
def contour_to_area(contours, model=None):
    
    data = check_model(model)
    
    size_correction = data["semi-polar coordinates"]["size correction"]
    alph, beta = [data["semi-polar coordinates"]["sagittal-area coefficients"][key]
                       for key in ["alpha", "beta"]]
    vp_map = data["semi-polar coordinates"]["map coeff"]
    iniva_tng = int(data["tongue"]["parameters"]["initial point"])
    c = size_correction * vp_map
    
    afs = [contour_to_area_function(contour, c, iniva_tng, alph, beta) 
              for contour in contours]
    
    AF = AreaFunction("area", np.array([x["area"] for x in afs]).T,
                      "length", np.array([x["length"] for x in afs]).T)

    AF.interpolate(40)
    return AF

def articulatory_to_area(articulatory_parameters, model=None):    
    contours = articulatory_to_contour(articulatory_parameters, model)
    area_function = contour_to_area(contours, model)
    return area_function

def articulatory_to_contour(articulatory_parameters, model=None):
    return vector_to_contours(articulatory_parameters, model)

def articulatory_to_formant(articulatory_parameters, model=None):
    return area_to_formant(articulatory_to_area(articulatory_parameters, 
                                                model=model))    

def articulatory_to_task(articulatory_parameters, model=None, verbosity=False):
    data = check_model(model)    
    size_correction = data["semi-polar coordinates"]["size correction"]    
    vp_map = data["semi-polar coordinates"]["map coeff"]
    c = size_correction * vp_map
    return contour_to_task(articulatory_to_contour(articulatory_parameters, model), 
                            c, verbosity=verbosity)

def articulatory_to_transfer_function(articulatory_parameters, model=None, 
                                      df=50):
    return area_to_transfer_function(articulatory_to_area(articulatory_parameters, 
                                                    model), df=df)

def contour_to_formant(contours, model=None):
    return area_to_formant(contour_to_area(contours, model))
    
def contour_to_task(contours, c, verbosity=False):
    if verbosity:
        from tqdm import tqdm
        return np.array([extract_task_variables(ctr, c) for ctr in tqdm(contours)]).T
    else:
        return np.array([extract_task_variables(ctr, c) for ctr in contours]).T

def contour_to_transfer_function(contours, model=None, df=50):
    return area_to_transfer_function(contour_to_area(contours, model), df=df)

def transfer_function_to_formant(transfer_function, freq):
    wvg = Waveguide("transfer_function", transfer_function,
                    "freq", freq)
    return wvg.computeformants()



        
    
    
    




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Feb  6 21:31:52 2019

@author: benjamin
"""

import os
import sys
import warnings
if not sys.warnoptions:
    warnings.simplefilter("ignore")
    os.environ["PYTHONWARNINGS"] = "ignore"
os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2' 

from ._forwardmapping import *
from .modeltools import *
from maedeep._areafunction import *
from maedeep._contour import *
from maedeep._waveguide import *



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Jul 29 09:58:38 2022

@author: benjamin
"""

import h5py
import os
from maedeep._config import get_maedeep_linear_model

def check_model(model):
    
    if model is None:
        model = get_maedeep_linear_model()
        
    if isinstance(model, str):
        if os.path.isfile(model):
            with h5py.File(model, "r") as file_id:
                return [file_id[key][()] for
                        key in ["matrix", "offset", "factor"]]
        else:
            raise ValueError(model + " is not a valid file")
    elif isinstance(model, dict):
        return model
    else:
        raise ValueError("The model is not a valid object")
    
def crop(x, lower_bound=None, upper_bound=None):
    if lower_bound is not None:
        x[x <= lower_bound] = lower_bound
    if upper_bound is not None:
        x[x >= upper_bound] = upper_bound
    return x
    
    
def normalize(x, offset, factor, direction=0):
    """ Normalization between 0 and 1"""
    if direction not in [0, 1]:
        raise ValueError("Direction not valid, please choose either 0 or 1")
    if direction == 0:
        return (x - offset)/factor
    elif direction == 1:
        return x * factor + offset



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Oct  6 09:32:12 2020

@author: benjamin
"""

import copy
import numpy as np
import scipy.interpolate as scintp

class AreaFunction:
    """class for manipulating area fucntions"""
    area = None
    length = None
    parent = None
    total_length = None
    constriction_area = None
    constriction_location = None

    def __init__(self, *args):
        nargin = len(args)
        for k in range(0, nargin, 2):
            key, value = args[k:k+2]
            setattr(self, key, value)
            
        if self.area is not None and self.length is not None:
            _, _, _ = self.characteristics()

    def copy(self):
        """returns a deep copy of the instance"""
        return copy.deepcopy(self)

    def interpolate(self, num_tubes):
        """interpolates area function into a vector y_out of N_out evenly spaced area function."""
        
        af = self.area
        lf = self.length
        if len(af.shape) == 1:
            af = af.reshape(-1, 1)
            lf = lf.reshape(-1, 1)

        cumx_in = np.cumsum(lf, 0)
        x_out = np.zeros((num_tubes, af.shape[1]))
        y_out = np.zeros((num_tubes, af.shape[1]))
        for kframe in range(af.shape[1]):
            y_in = af[:, kframe]
            x_in = cumx_in[:, kframe]

            x_out1 = np.arange(1, num_tubes+1)*x_in[-1]/num_tubes
            f1 = scintp.interp1d(x_in, y_in,
                                 kind='linear', fill_value='extrapolate')
            y_out[:, kframe] = f1(x_out1)
            x_out[:, kframe] = np.append(x_out1[0], np.diff(x_out1))

        self.area = y_out
        self.length = x_out
        return x_out, y_out
    
    def characteristics(self):
        """returns the characteristics of the area function"""
        
        self.total_length = np.sum(self.length, 0)
        self.constriction_area = np.min(self.area, 0)
        self.constriction_location = np.argmin(self.area, 0)
        
        return self.total_length, self.constriction_area, self.constriction_location
    
    def extend(self, nb_frame):
        """ Extends area function to a specified number of frames """
        if len(self.area.squeeze().shape) > 1:
            raise ValueError("Area function should contain only one frame")
            
        self.area = np.tile(self.area.squeeze().reshape(-1,1), 
                            (1, nb_frame))
        self.length = np.tile(self.length.squeeze().reshape(-1,1), 
                              (1, nb_frame))

def minimal_area(vt):
    
    lf, af = (vt[:,0], vt[:, 1])
    lf[lf<=0] = 1e-11
    af[af<=0] = 1e-11
    return af, lf




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Oct 30 07:53:09 2021

@author: benjamin
"""

from setuptools import setup, find_packages

setup(name='maedeep',
      version='0.0.6',
        description='Python interface to use articulatory model by Maeda',
        url='https://git.ecdf.ed.ac.uk/belie/maedeep',
        author='Benjamin Elie',
        author_email='benjamin.elie@ed.ac.uk',
        license='Creative Commons Attribution 4.0 International License',
      packages=find_packages(),
      install_requires=[
        "numpy",
        "scipy",
        "tqdm",
        "shapely",
        "tensorflow", 
        "keras",
        "scikit-learn",
        "pytest"
    ],
      include_package_data=True,
      package_data={'': ['data/*.h5', 'dnn/models/*.h5',
                         'linear/models/*.h5', 'parametric/models/*.json']},
      zip_safe=False)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon May 23 16:01:43 2022

@author: benjamin
"""

import os
from importlib.resources import files

def get_maedeep_path():
    return files("maedeep")#.as_posix()

def get_maedeep_model():
    return os.path.join(get_maedeep_path(), "parametric", "models", 
                        "model_spec.json")

def get_maedeep_linear_model():
    return os.path.join(get_maedeep_path(), "linear", "models", 
                        "linear_model.h5")

    
  